# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..errors.bad_request_error import BadRequestError
from ..errors.internal_server_error import InternalServerError
from ..errors.service_unavailable_error import ServiceUnavailableError
from ..errors.unauthorized_error import UnauthorizedError
from ..types.item_commodity_code import ItemCommodityCode
from ..types.item_description import ItemDescription
from ..types.item_product_code import ItemProductCode
from ..types.item_product_name import ItemProductName
from ..types.item_unitof_measure import ItemUnitofMeasure
from ..types.line_item_query_record import LineItemQueryRecord
from ..types.payabli_api_response import PayabliApiResponse
from ..types.payabli_api_response_6 import PayabliApiResponse6
from ..types.query_response_items import QueryResponseItems
from .types.delete_item_response import DeleteItemResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawLineItemClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def add_item(
        self,
        entry: str,
        *,
        item_cost: float,
        item_qty: int,
        idempotency_key: typing.Optional[str] = None,
        item_categories: typing.Optional[typing.Sequence[typing.Optional[str]]] = OMIT,
        item_commodity_code: typing.Optional[ItemCommodityCode] = OMIT,
        item_description: typing.Optional[ItemDescription] = OMIT,
        item_mode: typing.Optional[int] = OMIT,
        item_product_code: typing.Optional[ItemProductCode] = OMIT,
        item_product_name: typing.Optional[ItemProductName] = OMIT,
        item_unit_of_measure: typing.Optional[ItemUnitofMeasure] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[PayabliApiResponse6]:
        """
        Adds products and services to an entrypoint's catalog. These are used as line items for invoicing and transactions. In the response, "responseData" displays the item's code.

        Parameters
        ----------
        entry : str
            The paypoint's entrypoint identifier. [Learn more](/api-reference/api-overview#entrypoint-vs-entry)

        item_cost : float
            Item or product price per unit.

        item_qty : int
            Quantity of item or product.

        idempotency_key : typing.Optional[str]
            A unique ID you can include to prevent duplicating objects or transactions if a request is sent more than once. This key isn't generated in Payabli, you must generate it yourself.

        item_categories : typing.Optional[typing.Sequence[typing.Optional[str]]]
            Array of tags classifying item or product.

        item_commodity_code : typing.Optional[ItemCommodityCode]

        item_description : typing.Optional[ItemDescription]

        item_mode : typing.Optional[int]
            Internal class of item or product: value '0' is only for invoices, '1' for bills, and '2' is common for both.

        item_product_code : typing.Optional[ItemProductCode]

        item_product_name : typing.Optional[ItemProductName]

        item_unit_of_measure : typing.Optional[ItemUnitofMeasure]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PayabliApiResponse6]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"LineItem/{jsonable_encoder(entry)}",
            method="POST",
            json={
                "itemCategories": item_categories,
                "itemCommodityCode": item_commodity_code,
                "itemCost": item_cost,
                "itemDescription": item_description,
                "itemMode": item_mode,
                "itemProductCode": item_product_code,
                "itemProductName": item_product_name,
                "itemQty": item_qty,
                "itemUnitOfMeasure": item_unit_of_measure,
            },
            headers={
                "content-type": "application/json",
                "idempotencyKey": str(idempotency_key) if idempotency_key is not None else None,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PayabliApiResponse6,
                    parse_obj_as(
                        type_=PayabliApiResponse6,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete_item(
        self, line_item_id: int, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[DeleteItemResponse]:
        """
        Deletes an item.

        Parameters
        ----------
        line_item_id : int
            ID for the line item (also known as a product, service, or item).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[DeleteItemResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"LineItem/{jsonable_encoder(line_item_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DeleteItemResponse,
                    parse_obj_as(
                        type_=DeleteItemResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_item(
        self, line_item_id: int, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[LineItemQueryRecord]:
        """
        Gets an item by ID.

        Parameters
        ----------
        line_item_id : int
            ID for the line item (also known as a product, service, or item).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[LineItemQueryRecord]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"LineItem/{jsonable_encoder(line_item_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    LineItemQueryRecord,
                    parse_obj_as(
                        type_=LineItemQueryRecord,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def list_line_items(
        self,
        entry: str,
        *,
        from_record: typing.Optional[int] = None,
        limit_record: typing.Optional[int] = None,
        parameters: typing.Optional[typing.Dict[str, typing.Optional[str]]] = None,
        sort_by: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[QueryResponseItems]:
        """
        Retrieves a list of line items and their details from an entrypoint. Line items are also known as items, products, and services. Use filters to limit results.

        Parameters
        ----------
        entry : str
            The paypoint's entrypoint identifier. [Learn more](/api-reference/api-overview#entrypoint-vs-entry)

        from_record : typing.Optional[int]
            The number of records to skip before starting to collect the result set.

        limit_record : typing.Optional[int]
            Max number of records to return for the query. Use `0` or negative value to return all records.

        parameters : typing.Optional[typing.Dict[str, typing.Optional[str]]]

            Collection of field names, conditions, and values used to filter the query
            <Info>
              **You must remove `parameters=` from the request before you send it, otherwise Payabli will ignore the filters.**

              Because of a technical limitation, you can't make a request that includes filters from the API console on this page. The response won't be filtered. Instead, copy the request, remove `parameters=` and run the request in a different client.

              For example:

              --url https://api-sandbox.payabli.com/api/Query/transactions/org/236?parameters=totalAmount(gt)=1000&limitRecord=20

              should become:

              --url https://api-sandbox.payabli.com/api/Query/transactions/org/236?totalAmount(gt)=1000&limitRecord=20

            </Info>
            See [Filters and Conditions Reference](/developers/developer-guides/pay-ops-reporting-engine-overview#filters-and-conditions-reference) for help.

            List of field names accepted:

              - `categories` (ct, nct)
              - `code` (ne, eq, ct, nct)
              - `commodityCode` (ne, eq, ct, nct)
              - `createdDate` (gt, ge, lt, le, eq, ne)
              - `description` (ne, eq, ct, nct)
              - `externalPaypointID` (ct, nct, ne, eq)
              - `mode` (eq, ne)
              - `name` (ne, eq, ct, nct)
              - `orgName` (ne, eq, ct, nct)
              - `paypointDba` (ne, eq, ct, nct)
              - `paypointId` (ne, eq)
              - `paypointLegal` (ne, eq, ct, nct)
              - `quantity` (gt, ge, lt, le, eq, ne)
              - `uom` (ne, eq, ct, nct)
              - `updatedDate` (gt, ge, lt, le, eq, ne)
              - `value` (gt, ge, lt, le, eq, ne)

            List of comparison accepted - enclosed between parentheses:

            - eq or empty => equal
            - gt => greater than
            - ge => greater or equal
            - lt => less than
            - le => less or equal
            - ne => not equal
            - ct => contains
            - nct => not contains
            - in => inside array separated by "|"
            - nin => not inside array separated by "|"

            List of parameters accepted:
            - limitRecord : max number of records for query (default="20", "0" or negative value for all)
            - fromRecord : initial record in query

            Example: name(ct)=john return all records with name containing john

        sort_by : typing.Optional[str]
            The field name to use for sorting results. Use `desc(field_name)` to sort descending by `field_name`, and use `asc(field_name)` to sort ascending by `field_name`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[QueryResponseItems]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"Query/lineitems/{jsonable_encoder(entry)}",
            method="GET",
            params={
                "fromRecord": from_record,
                "limitRecord": limit_record,
                "parameters": parameters,
                "sortBy": sort_by,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    QueryResponseItems,
                    parse_obj_as(
                        type_=QueryResponseItems,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update_item(
        self,
        line_item_id: int,
        *,
        item_cost: float,
        item_qty: int,
        item_categories: typing.Optional[typing.Sequence[typing.Optional[str]]] = OMIT,
        item_commodity_code: typing.Optional[ItemCommodityCode] = OMIT,
        item_description: typing.Optional[ItemDescription] = OMIT,
        item_mode: typing.Optional[int] = OMIT,
        item_product_code: typing.Optional[ItemProductCode] = OMIT,
        item_product_name: typing.Optional[ItemProductName] = OMIT,
        item_unit_of_measure: typing.Optional[ItemUnitofMeasure] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[PayabliApiResponse6]:
        """
        Updates an item.

        Parameters
        ----------
        line_item_id : int
            ID for the line item (also known as a product, service, or item).

        item_cost : float
            Item or product price per unit.

        item_qty : int
            Quantity of item or product.

        item_categories : typing.Optional[typing.Sequence[typing.Optional[str]]]
            Array of tags classifying item or product.

        item_commodity_code : typing.Optional[ItemCommodityCode]

        item_description : typing.Optional[ItemDescription]

        item_mode : typing.Optional[int]
            Internal class of item or product: value '0' is only for invoices, '1' for bills, and '2' is common for both.

        item_product_code : typing.Optional[ItemProductCode]

        item_product_name : typing.Optional[ItemProductName]

        item_unit_of_measure : typing.Optional[ItemUnitofMeasure]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PayabliApiResponse6]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"LineItem/{jsonable_encoder(line_item_id)}",
            method="PUT",
            json={
                "itemCategories": item_categories,
                "itemCommodityCode": item_commodity_code,
                "itemCost": item_cost,
                "itemDescription": item_description,
                "itemMode": item_mode,
                "itemProductCode": item_product_code,
                "itemProductName": item_product_name,
                "itemQty": item_qty,
                "itemUnitOfMeasure": item_unit_of_measure,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PayabliApiResponse6,
                    parse_obj_as(
                        type_=PayabliApiResponse6,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawLineItemClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def add_item(
        self,
        entry: str,
        *,
        item_cost: float,
        item_qty: int,
        idempotency_key: typing.Optional[str] = None,
        item_categories: typing.Optional[typing.Sequence[typing.Optional[str]]] = OMIT,
        item_commodity_code: typing.Optional[ItemCommodityCode] = OMIT,
        item_description: typing.Optional[ItemDescription] = OMIT,
        item_mode: typing.Optional[int] = OMIT,
        item_product_code: typing.Optional[ItemProductCode] = OMIT,
        item_product_name: typing.Optional[ItemProductName] = OMIT,
        item_unit_of_measure: typing.Optional[ItemUnitofMeasure] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[PayabliApiResponse6]:
        """
        Adds products and services to an entrypoint's catalog. These are used as line items for invoicing and transactions. In the response, "responseData" displays the item's code.

        Parameters
        ----------
        entry : str
            The paypoint's entrypoint identifier. [Learn more](/api-reference/api-overview#entrypoint-vs-entry)

        item_cost : float
            Item or product price per unit.

        item_qty : int
            Quantity of item or product.

        idempotency_key : typing.Optional[str]
            A unique ID you can include to prevent duplicating objects or transactions if a request is sent more than once. This key isn't generated in Payabli, you must generate it yourself.

        item_categories : typing.Optional[typing.Sequence[typing.Optional[str]]]
            Array of tags classifying item or product.

        item_commodity_code : typing.Optional[ItemCommodityCode]

        item_description : typing.Optional[ItemDescription]

        item_mode : typing.Optional[int]
            Internal class of item or product: value '0' is only for invoices, '1' for bills, and '2' is common for both.

        item_product_code : typing.Optional[ItemProductCode]

        item_product_name : typing.Optional[ItemProductName]

        item_unit_of_measure : typing.Optional[ItemUnitofMeasure]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PayabliApiResponse6]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"LineItem/{jsonable_encoder(entry)}",
            method="POST",
            json={
                "itemCategories": item_categories,
                "itemCommodityCode": item_commodity_code,
                "itemCost": item_cost,
                "itemDescription": item_description,
                "itemMode": item_mode,
                "itemProductCode": item_product_code,
                "itemProductName": item_product_name,
                "itemQty": item_qty,
                "itemUnitOfMeasure": item_unit_of_measure,
            },
            headers={
                "content-type": "application/json",
                "idempotencyKey": str(idempotency_key) if idempotency_key is not None else None,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PayabliApiResponse6,
                    parse_obj_as(
                        type_=PayabliApiResponse6,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete_item(
        self, line_item_id: int, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[DeleteItemResponse]:
        """
        Deletes an item.

        Parameters
        ----------
        line_item_id : int
            ID for the line item (also known as a product, service, or item).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[DeleteItemResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"LineItem/{jsonable_encoder(line_item_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DeleteItemResponse,
                    parse_obj_as(
                        type_=DeleteItemResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_item(
        self, line_item_id: int, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[LineItemQueryRecord]:
        """
        Gets an item by ID.

        Parameters
        ----------
        line_item_id : int
            ID for the line item (also known as a product, service, or item).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[LineItemQueryRecord]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"LineItem/{jsonable_encoder(line_item_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    LineItemQueryRecord,
                    parse_obj_as(
                        type_=LineItemQueryRecord,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def list_line_items(
        self,
        entry: str,
        *,
        from_record: typing.Optional[int] = None,
        limit_record: typing.Optional[int] = None,
        parameters: typing.Optional[typing.Dict[str, typing.Optional[str]]] = None,
        sort_by: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[QueryResponseItems]:
        """
        Retrieves a list of line items and their details from an entrypoint. Line items are also known as items, products, and services. Use filters to limit results.

        Parameters
        ----------
        entry : str
            The paypoint's entrypoint identifier. [Learn more](/api-reference/api-overview#entrypoint-vs-entry)

        from_record : typing.Optional[int]
            The number of records to skip before starting to collect the result set.

        limit_record : typing.Optional[int]
            Max number of records to return for the query. Use `0` or negative value to return all records.

        parameters : typing.Optional[typing.Dict[str, typing.Optional[str]]]

            Collection of field names, conditions, and values used to filter the query
            <Info>
              **You must remove `parameters=` from the request before you send it, otherwise Payabli will ignore the filters.**

              Because of a technical limitation, you can't make a request that includes filters from the API console on this page. The response won't be filtered. Instead, copy the request, remove `parameters=` and run the request in a different client.

              For example:

              --url https://api-sandbox.payabli.com/api/Query/transactions/org/236?parameters=totalAmount(gt)=1000&limitRecord=20

              should become:

              --url https://api-sandbox.payabli.com/api/Query/transactions/org/236?totalAmount(gt)=1000&limitRecord=20

            </Info>
            See [Filters and Conditions Reference](/developers/developer-guides/pay-ops-reporting-engine-overview#filters-and-conditions-reference) for help.

            List of field names accepted:

              - `categories` (ct, nct)
              - `code` (ne, eq, ct, nct)
              - `commodityCode` (ne, eq, ct, nct)
              - `createdDate` (gt, ge, lt, le, eq, ne)
              - `description` (ne, eq, ct, nct)
              - `externalPaypointID` (ct, nct, ne, eq)
              - `mode` (eq, ne)
              - `name` (ne, eq, ct, nct)
              - `orgName` (ne, eq, ct, nct)
              - `paypointDba` (ne, eq, ct, nct)
              - `paypointId` (ne, eq)
              - `paypointLegal` (ne, eq, ct, nct)
              - `quantity` (gt, ge, lt, le, eq, ne)
              - `uom` (ne, eq, ct, nct)
              - `updatedDate` (gt, ge, lt, le, eq, ne)
              - `value` (gt, ge, lt, le, eq, ne)

            List of comparison accepted - enclosed between parentheses:

            - eq or empty => equal
            - gt => greater than
            - ge => greater or equal
            - lt => less than
            - le => less or equal
            - ne => not equal
            - ct => contains
            - nct => not contains
            - in => inside array separated by "|"
            - nin => not inside array separated by "|"

            List of parameters accepted:
            - limitRecord : max number of records for query (default="20", "0" or negative value for all)
            - fromRecord : initial record in query

            Example: name(ct)=john return all records with name containing john

        sort_by : typing.Optional[str]
            The field name to use for sorting results. Use `desc(field_name)` to sort descending by `field_name`, and use `asc(field_name)` to sort ascending by `field_name`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[QueryResponseItems]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"Query/lineitems/{jsonable_encoder(entry)}",
            method="GET",
            params={
                "fromRecord": from_record,
                "limitRecord": limit_record,
                "parameters": parameters,
                "sortBy": sort_by,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    QueryResponseItems,
                    parse_obj_as(
                        type_=QueryResponseItems,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update_item(
        self,
        line_item_id: int,
        *,
        item_cost: float,
        item_qty: int,
        item_categories: typing.Optional[typing.Sequence[typing.Optional[str]]] = OMIT,
        item_commodity_code: typing.Optional[ItemCommodityCode] = OMIT,
        item_description: typing.Optional[ItemDescription] = OMIT,
        item_mode: typing.Optional[int] = OMIT,
        item_product_code: typing.Optional[ItemProductCode] = OMIT,
        item_product_name: typing.Optional[ItemProductName] = OMIT,
        item_unit_of_measure: typing.Optional[ItemUnitofMeasure] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[PayabliApiResponse6]:
        """
        Updates an item.

        Parameters
        ----------
        line_item_id : int
            ID for the line item (also known as a product, service, or item).

        item_cost : float
            Item or product price per unit.

        item_qty : int
            Quantity of item or product.

        item_categories : typing.Optional[typing.Sequence[typing.Optional[str]]]
            Array of tags classifying item or product.

        item_commodity_code : typing.Optional[ItemCommodityCode]

        item_description : typing.Optional[ItemDescription]

        item_mode : typing.Optional[int]
            Internal class of item or product: value '0' is only for invoices, '1' for bills, and '2' is common for both.

        item_product_code : typing.Optional[ItemProductCode]

        item_product_name : typing.Optional[ItemProductName]

        item_unit_of_measure : typing.Optional[ItemUnitofMeasure]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PayabliApiResponse6]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"LineItem/{jsonable_encoder(line_item_id)}",
            method="PUT",
            json={
                "itemCategories": item_categories,
                "itemCommodityCode": item_commodity_code,
                "itemCost": item_cost,
                "itemDescription": item_description,
                "itemMode": item_mode,
                "itemProductCode": item_product_code,
                "itemProductName": item_product_name,
                "itemQty": item_qty,
                "itemUnitOfMeasure": item_unit_of_measure,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PayabliApiResponse6,
                    parse_obj_as(
                        type_=PayabliApiResponse6,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..errors.bad_request_error import BadRequestError
from ..errors.internal_server_error import InternalServerError
from ..errors.service_unavailable_error import ServiceUnavailableError
from ..errors.unauthorized_error import UnauthorizedError
from ..types.file_content_ftype import FileContentFtype
from ..types.payabli_api_response import PayabliApiResponse
from .types.payabli_api_response_ocr import PayabliApiResponseOcr
from .types.type_result import TypeResult

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawOcrClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def ocr_document_form(
        self,
        type_result: TypeResult,
        *,
        ftype: typing.Optional[FileContentFtype] = OMIT,
        filename: typing.Optional[str] = OMIT,
        furl: typing.Optional[str] = OMIT,
        f_content: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[PayabliApiResponseOcr]:
        """
        Parameters
        ----------
        type_result : TypeResult

        ftype : typing.Optional[FileContentFtype]

        filename : typing.Optional[str]
            The name of the file to be uploaded

        furl : typing.Optional[str]
            Optional URL link to the file

        f_content : typing.Optional[str]
            Base64-encoded file content

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PayabliApiResponseOcr]
            OCR document processing result
        """
        _response = self._client_wrapper.httpx_client.request(
            f"Import/ocrDocumentForm/{jsonable_encoder(type_result)}",
            method="POST",
            json={
                "ftype": ftype,
                "filename": filename,
                "furl": furl,
                "fContent": f_content,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PayabliApiResponseOcr,
                    parse_obj_as(
                        type_=PayabliApiResponseOcr,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def ocr_document_json(
        self,
        type_result: TypeResult,
        *,
        ftype: typing.Optional[FileContentFtype] = OMIT,
        filename: typing.Optional[str] = OMIT,
        furl: typing.Optional[str] = OMIT,
        f_content: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[PayabliApiResponseOcr]:
        """
        Parameters
        ----------
        type_result : TypeResult

        ftype : typing.Optional[FileContentFtype]

        filename : typing.Optional[str]
            The name of the file to be uploaded

        furl : typing.Optional[str]
            Optional URL link to the file

        f_content : typing.Optional[str]
            Base64-encoded file content

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PayabliApiResponseOcr]
            OCR document processing result
        """
        _response = self._client_wrapper.httpx_client.request(
            f"Import/ocrDocumentJson/{jsonable_encoder(type_result)}",
            method="POST",
            json={
                "ftype": ftype,
                "filename": filename,
                "furl": furl,
                "fContent": f_content,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PayabliApiResponseOcr,
                    parse_obj_as(
                        type_=PayabliApiResponseOcr,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawOcrClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def ocr_document_form(
        self,
        type_result: TypeResult,
        *,
        ftype: typing.Optional[FileContentFtype] = OMIT,
        filename: typing.Optional[str] = OMIT,
        furl: typing.Optional[str] = OMIT,
        f_content: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[PayabliApiResponseOcr]:
        """
        Parameters
        ----------
        type_result : TypeResult

        ftype : typing.Optional[FileContentFtype]

        filename : typing.Optional[str]
            The name of the file to be uploaded

        furl : typing.Optional[str]
            Optional URL link to the file

        f_content : typing.Optional[str]
            Base64-encoded file content

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PayabliApiResponseOcr]
            OCR document processing result
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"Import/ocrDocumentForm/{jsonable_encoder(type_result)}",
            method="POST",
            json={
                "ftype": ftype,
                "filename": filename,
                "furl": furl,
                "fContent": f_content,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PayabliApiResponseOcr,
                    parse_obj_as(
                        type_=PayabliApiResponseOcr,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def ocr_document_json(
        self,
        type_result: TypeResult,
        *,
        ftype: typing.Optional[FileContentFtype] = OMIT,
        filename: typing.Optional[str] = OMIT,
        furl: typing.Optional[str] = OMIT,
        f_content: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[PayabliApiResponseOcr]:
        """
        Parameters
        ----------
        type_result : TypeResult

        ftype : typing.Optional[FileContentFtype]

        filename : typing.Optional[str]
            The name of the file to be uploaded

        furl : typing.Optional[str]
            Optional URL link to the file

        f_content : typing.Optional[str]
            Base64-encoded file content

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PayabliApiResponseOcr]
            OCR document processing result
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"Import/ocrDocumentJson/{jsonable_encoder(type_result)}",
            method="POST",
            json={
                "ftype": ftype,
                "filename": filename,
                "furl": furl,
                "fContent": f_content,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PayabliApiResponseOcr,
                    parse_obj_as(
                        type_=PayabliApiResponseOcr,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .tier_item import TierItem


class CardType(UniversalBaseModel):
    amex: typing.Optional[TierItem] = None
    discover: typing.Optional[TierItem] = None
    master_card: typing_extensions.Annotated[typing.Optional[TierItem], FieldMetadata(alias="masterCard")] = None
    visa: typing.Optional[TierItem] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from ...types.customer_id import CustomerId
from ...types.is_success import IsSuccess
from ...types.response_text import ResponseText


class AddSubscriptionResponse(UniversalBaseModel):
    """
    Success response

    Examples
    --------
    from payabli.subscription import AddSubscriptionResponse

    AddSubscriptionResponse(
        response_text="Success",
        is_success=True,
        response_data=396,
        customer_id=4440,
    )
    """

    customer_id: typing_extensions.Annotated[typing.Optional[CustomerId], FieldMetadata(alias="customerId")] = None
    response_text: typing_extensions.Annotated[ResponseText, FieldMetadata(alias="responseText")]
    is_success: typing_extensions.Annotated[typing.Optional[IsSuccess], FieldMetadata(alias="isSuccess")] = None
    response_data: typing_extensions.Annotated[int, FieldMetadata(alias="responseData")] = pydantic.Field()
    """
    The identifier of the newly created subscription.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..core.serialization import convert_and_respect_annotation_metadata
from ..errors.bad_request_error import BadRequestError
from ..errors.internal_server_error import InternalServerError
from ..errors.service_unavailable_error import ServiceUnavailableError
from ..errors.unauthorized_error import UnauthorizedError
from ..types.additional_data import AdditionalData
from ..types.email import Email
from ..types.language import Language
from ..types.mfa_data import MfaData
from ..types.mfa_mode import MfaMode
from ..types.mfa_validation_code import MfaValidationCode
from ..types.name_user import NameUser
from ..types.org_scope import OrgScope
from ..types.payabli_api_response import PayabliApiResponse
from ..types.payabli_api_response_mfa_basic import PayabliApiResponseMfaBasic
from ..types.payabli_api_response_user_mfa import PayabliApiResponseUserMfa
from ..types.phone_number import PhoneNumber
from ..types.timezone import Timezone
from ..types.user_query_record import UserQueryRecord
from ..types.usr_access import UsrAccess
from ..types.usr_status import UsrStatus
from .types.add_user_response import AddUserResponse
from .types.auth_reset_user_response import AuthResetUserResponse
from .types.change_psw_user_response import ChangePswUserResponse
from .types.delete_user_response import DeleteUserResponse
from .types.edit_mfa_user_response import EditMfaUserResponse
from .types.logout_user_response import LogoutUserResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawUserClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def add_user(
        self,
        *,
        access: typing.Optional[typing.Sequence[UsrAccess]] = OMIT,
        additional_data: typing.Optional[AdditionalData] = OMIT,
        email: typing.Optional[Email] = OMIT,
        language: typing.Optional[Language] = OMIT,
        mfa_data: typing.Optional[MfaData] = OMIT,
        name: typing.Optional[NameUser] = OMIT,
        phone: typing.Optional[PhoneNumber] = OMIT,
        pwd: typing.Optional[str] = OMIT,
        scope: typing.Optional[typing.Sequence[OrgScope]] = OMIT,
        time_zone: typing.Optional[Timezone] = OMIT,
        usr_status: typing.Optional[UsrStatus] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[AddUserResponse]:
        """
        Parameters
        ----------
        access : typing.Optional[typing.Sequence[UsrAccess]]

        additional_data : typing.Optional[AdditionalData]

        email : typing.Optional[Email]
            The user's email address.

        language : typing.Optional[Language]

        mfa_data : typing.Optional[MfaData]

        name : typing.Optional[NameUser]

        phone : typing.Optional[PhoneNumber]
            The user's phone number.

        pwd : typing.Optional[str]

        scope : typing.Optional[typing.Sequence[OrgScope]]

        time_zone : typing.Optional[Timezone]

        usr_status : typing.Optional[UsrStatus]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[AddUserResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "User",
            method="POST",
            json={
                "access": convert_and_respect_annotation_metadata(
                    object_=access, annotation=typing.Sequence[UsrAccess], direction="write"
                ),
                "additionalData": additional_data,
                "email": email,
                "language": language,
                "mfaData": convert_and_respect_annotation_metadata(
                    object_=mfa_data, annotation=MfaData, direction="write"
                ),
                "name": name,
                "phone": phone,
                "pwd": pwd,
                "scope": convert_and_respect_annotation_metadata(
                    object_=scope, annotation=typing.Sequence[OrgScope], direction="write"
                ),
                "timeZone": time_zone,
                "usrStatus": usr_status,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AddUserResponse,
                    parse_obj_as(
                        type_=AddUserResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def auth_refresh_user(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[PayabliApiResponseUserMfa]:
        """
        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PayabliApiResponseUserMfa]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "User/authrefresh",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PayabliApiResponseUserMfa,
                    parse_obj_as(
                        type_=PayabliApiResponseUserMfa,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def auth_reset_user(
        self,
        *,
        email: typing.Optional[Email] = OMIT,
        entry: typing.Optional[str] = OMIT,
        entry_type: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[AuthResetUserResponse]:
        """
        Parameters
        ----------
        email : typing.Optional[Email]
            The user's email address.

        entry : typing.Optional[str]
            Identifier for entrypoint originating the request (used by front-end apps)

        entry_type : typing.Optional[int]
            Type of entry identifier: 0 - partner, 2 - paypoint. This is used by front-end apps, required if an Entry is indicated.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[AuthResetUserResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "User/authreset",
            method="POST",
            json={
                "email": email,
                "entry": entry,
                "entryType": entry_type,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AuthResetUserResponse,
                    parse_obj_as(
                        type_=AuthResetUserResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def auth_user(
        self,
        provider: str,
        *,
        email: typing.Optional[Email] = OMIT,
        entry: typing.Optional[str] = OMIT,
        entry_type: typing.Optional[int] = OMIT,
        psw: typing.Optional[str] = OMIT,
        user_id: typing.Optional[int] = OMIT,
        user_token_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[PayabliApiResponseMfaBasic]:
        """
        This endpoint requires an application API token.

        Parameters
        ----------
        provider : str
            Auth provider. This fields is optional and defaults to null for the built-in provider.

        email : typing.Optional[Email]

        entry : typing.Optional[str]
            Identifier for entry point originating the request (used by front-end apps)

        entry_type : typing.Optional[int]
            Type of entry identifier: 0 - partner, 2 - paypoint. This is used by front-end apps, required if an Entry is indicated.

        psw : typing.Optional[str]

        user_id : typing.Optional[int]

        user_token_id : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PayabliApiResponseMfaBasic]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"User/auth/{jsonable_encoder(provider)}",
            method="POST",
            json={
                "email": email,
                "entry": entry,
                "entryType": entry_type,
                "psw": psw,
                "userId": user_id,
                "userTokenId": user_token_id,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PayabliApiResponseMfaBasic,
                    parse_obj_as(
                        type_=PayabliApiResponseMfaBasic,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def change_psw_user(
        self, *, psw: typing.Optional[str] = OMIT, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[ChangePswUserResponse]:
        """
        Parameters
        ----------
        psw : typing.Optional[str]
            New User password

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ChangePswUserResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "User/authpsw",
            method="PUT",
            json={
                "psw": psw,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ChangePswUserResponse,
                    parse_obj_as(
                        type_=ChangePswUserResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete_user(
        self, user_id: int, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[DeleteUserResponse]:
        """
        Parameters
        ----------
        user_id : int
            The Payabli-generated `userId` value.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[DeleteUserResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"User/{jsonable_encoder(user_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DeleteUserResponse,
                    parse_obj_as(
                        type_=DeleteUserResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def edit_mfa_user(
        self,
        user_id: int,
        *,
        mfa: typing.Optional[bool] = OMIT,
        mfa_mode: typing.Optional[MfaMode] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[EditMfaUserResponse]:
        """
        Parameters
        ----------
        user_id : int
            User Identifier

        mfa : typing.Optional[bool]

        mfa_mode : typing.Optional[MfaMode]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[EditMfaUserResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"User/mfa/{jsonable_encoder(user_id)}",
            method="PUT",
            json={
                "mfa": mfa,
                "mfaMode": mfa_mode,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    EditMfaUserResponse,
                    parse_obj_as(
                        type_=EditMfaUserResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def edit_user(
        self,
        user_id: int,
        *,
        access: typing.Optional[typing.Sequence[UsrAccess]] = OMIT,
        additional_data: typing.Optional[AdditionalData] = OMIT,
        email: typing.Optional[Email] = OMIT,
        language: typing.Optional[Language] = OMIT,
        mfa_data: typing.Optional[MfaData] = OMIT,
        name: typing.Optional[NameUser] = OMIT,
        phone: typing.Optional[PhoneNumber] = OMIT,
        pwd: typing.Optional[str] = OMIT,
        scope: typing.Optional[typing.Sequence[OrgScope]] = OMIT,
        time_zone: typing.Optional[Timezone] = OMIT,
        usr_status: typing.Optional[UsrStatus] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[PayabliApiResponse]:
        """
        Parameters
        ----------
        user_id : int
            User Identifier

        access : typing.Optional[typing.Sequence[UsrAccess]]

        additional_data : typing.Optional[AdditionalData]

        email : typing.Optional[Email]
            The user's email address.

        language : typing.Optional[Language]

        mfa_data : typing.Optional[MfaData]

        name : typing.Optional[NameUser]

        phone : typing.Optional[PhoneNumber]
            The user's phone number.

        pwd : typing.Optional[str]

        scope : typing.Optional[typing.Sequence[OrgScope]]

        time_zone : typing.Optional[Timezone]

        usr_status : typing.Optional[UsrStatus]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PayabliApiResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"User/{jsonable_encoder(user_id)}",
            method="PUT",
            json={
                "access": convert_and_respect_annotation_metadata(
                    object_=access, annotation=typing.Sequence[UsrAccess], direction="write"
                ),
                "additionalData": additional_data,
                "email": email,
                "language": language,
                "mfaData": convert_and_respect_annotation_metadata(
                    object_=mfa_data, annotation=MfaData, direction="write"
                ),
                "name": name,
                "phone": phone,
                "pwd": pwd,
                "scope": convert_and_respect_annotation_metadata(
                    object_=scope, annotation=typing.Sequence[OrgScope], direction="write"
                ),
                "timeZone": time_zone,
                "usrStatus": usr_status,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PayabliApiResponse,
                    parse_obj_as(
                        type_=PayabliApiResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_user(
        self,
        user_id: int,
        *,
        entry: typing.Optional[str] = None,
        level: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[UserQueryRecord]:
        """
        Parameters
        ----------
        user_id : int
            The Payabli-generated `userId` value.

        entry : typing.Optional[str]
            The entrypoint identifier.

        level : typing.Optional[int]
            Entry level: 0 - partner, 2 - paypoint

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[UserQueryRecord]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"User/{jsonable_encoder(user_id)}",
            method="GET",
            params={
                "entry": entry,
                "level": level,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UserQueryRecord,
                    parse_obj_as(
                        type_=UserQueryRecord,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def logout_user(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[LogoutUserResponse]:
        """
        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[LogoutUserResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "User/authlogout",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    LogoutUserResponse,
                    parse_obj_as(
                        type_=LogoutUserResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def resend_mfa_code(
        self, entry: str, entry_type: int, usrname: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[PayabliApiResponseMfaBasic]:
        """
        Parameters
        ----------
        entry : str


        entry_type : int


        usrname : str


        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PayabliApiResponseMfaBasic]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"User/resendmfa/{jsonable_encoder(usrname)}/{jsonable_encoder(entry)}/{jsonable_encoder(entry_type)}",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PayabliApiResponseMfaBasic,
                    parse_obj_as(
                        type_=PayabliApiResponseMfaBasic,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def validate_mfa_user(
        self,
        *,
        mfa_code: typing.Optional[str] = OMIT,
        mfa_validation_code: typing.Optional[MfaValidationCode] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[PayabliApiResponseUserMfa]:
        """
        Parameters
        ----------
        mfa_code : typing.Optional[str]

        mfa_validation_code : typing.Optional[MfaValidationCode]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PayabliApiResponseUserMfa]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "User/mfa",
            method="POST",
            json={
                "mfaCode": mfa_code,
                "mfaValidationCode": mfa_validation_code,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PayabliApiResponseUserMfa,
                    parse_obj_as(
                        type_=PayabliApiResponseUserMfa,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawUserClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def add_user(
        self,
        *,
        access: typing.Optional[typing.Sequence[UsrAccess]] = OMIT,
        additional_data: typing.Optional[AdditionalData] = OMIT,
        email: typing.Optional[Email] = OMIT,
        language: typing.Optional[Language] = OMIT,
        mfa_data: typing.Optional[MfaData] = OMIT,
        name: typing.Optional[NameUser] = OMIT,
        phone: typing.Optional[PhoneNumber] = OMIT,
        pwd: typing.Optional[str] = OMIT,
        scope: typing.Optional[typing.Sequence[OrgScope]] = OMIT,
        time_zone: typing.Optional[Timezone] = OMIT,
        usr_status: typing.Optional[UsrStatus] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[AddUserResponse]:
        """
        Parameters
        ----------
        access : typing.Optional[typing.Sequence[UsrAccess]]

        additional_data : typing.Optional[AdditionalData]

        email : typing.Optional[Email]
            The user's email address.

        language : typing.Optional[Language]

        mfa_data : typing.Optional[MfaData]

        name : typing.Optional[NameUser]

        phone : typing.Optional[PhoneNumber]
            The user's phone number.

        pwd : typing.Optional[str]

        scope : typing.Optional[typing.Sequence[OrgScope]]

        time_zone : typing.Optional[Timezone]

        usr_status : typing.Optional[UsrStatus]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[AddUserResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "User",
            method="POST",
            json={
                "access": convert_and_respect_annotation_metadata(
                    object_=access, annotation=typing.Sequence[UsrAccess], direction="write"
                ),
                "additionalData": additional_data,
                "email": email,
                "language": language,
                "mfaData": convert_and_respect_annotation_metadata(
                    object_=mfa_data, annotation=MfaData, direction="write"
                ),
                "name": name,
                "phone": phone,
                "pwd": pwd,
                "scope": convert_and_respect_annotation_metadata(
                    object_=scope, annotation=typing.Sequence[OrgScope], direction="write"
                ),
                "timeZone": time_zone,
                "usrStatus": usr_status,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AddUserResponse,
                    parse_obj_as(
                        type_=AddUserResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def auth_refresh_user(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[PayabliApiResponseUserMfa]:
        """
        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PayabliApiResponseUserMfa]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "User/authrefresh",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PayabliApiResponseUserMfa,
                    parse_obj_as(
                        type_=PayabliApiResponseUserMfa,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def auth_reset_user(
        self,
        *,
        email: typing.Optional[Email] = OMIT,
        entry: typing.Optional[str] = OMIT,
        entry_type: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[AuthResetUserResponse]:
        """
        Parameters
        ----------
        email : typing.Optional[Email]
            The user's email address.

        entry : typing.Optional[str]
            Identifier for entrypoint originating the request (used by front-end apps)

        entry_type : typing.Optional[int]
            Type of entry identifier: 0 - partner, 2 - paypoint. This is used by front-end apps, required if an Entry is indicated.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[AuthResetUserResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "User/authreset",
            method="POST",
            json={
                "email": email,
                "entry": entry,
                "entryType": entry_type,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AuthResetUserResponse,
                    parse_obj_as(
                        type_=AuthResetUserResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def auth_user(
        self,
        provider: str,
        *,
        email: typing.Optional[Email] = OMIT,
        entry: typing.Optional[str] = OMIT,
        entry_type: typing.Optional[int] = OMIT,
        psw: typing.Optional[str] = OMIT,
        user_id: typing.Optional[int] = OMIT,
        user_token_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[PayabliApiResponseMfaBasic]:
        """
        This endpoint requires an application API token.

        Parameters
        ----------
        provider : str
            Auth provider. This fields is optional and defaults to null for the built-in provider.

        email : typing.Optional[Email]

        entry : typing.Optional[str]
            Identifier for entry point originating the request (used by front-end apps)

        entry_type : typing.Optional[int]
            Type of entry identifier: 0 - partner, 2 - paypoint. This is used by front-end apps, required if an Entry is indicated.

        psw : typing.Optional[str]

        user_id : typing.Optional[int]

        user_token_id : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PayabliApiResponseMfaBasic]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"User/auth/{jsonable_encoder(provider)}",
            method="POST",
            json={
                "email": email,
                "entry": entry,
                "entryType": entry_type,
                "psw": psw,
                "userId": user_id,
                "userTokenId": user_token_id,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PayabliApiResponseMfaBasic,
                    parse_obj_as(
                        type_=PayabliApiResponseMfaBasic,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def change_psw_user(
        self, *, psw: typing.Optional[str] = OMIT, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[ChangePswUserResponse]:
        """
        Parameters
        ----------
        psw : typing.Optional[str]
            New User password

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ChangePswUserResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "User/authpsw",
            method="PUT",
            json={
                "psw": psw,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ChangePswUserResponse,
                    parse_obj_as(
                        type_=ChangePswUserResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete_user(
        self, user_id: int, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[DeleteUserResponse]:
        """
        Parameters
        ----------
        user_id : int
            The Payabli-generated `userId` value.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[DeleteUserResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"User/{jsonable_encoder(user_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DeleteUserResponse,
                    parse_obj_as(
                        type_=DeleteUserResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def edit_mfa_user(
        self,
        user_id: int,
        *,
        mfa: typing.Optional[bool] = OMIT,
        mfa_mode: typing.Optional[MfaMode] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[EditMfaUserResponse]:
        """
        Parameters
        ----------
        user_id : int
            User Identifier

        mfa : typing.Optional[bool]

        mfa_mode : typing.Optional[MfaMode]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[EditMfaUserResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"User/mfa/{jsonable_encoder(user_id)}",
            method="PUT",
            json={
                "mfa": mfa,
                "mfaMode": mfa_mode,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    EditMfaUserResponse,
                    parse_obj_as(
                        type_=EditMfaUserResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def edit_user(
        self,
        user_id: int,
        *,
        access: typing.Optional[typing.Sequence[UsrAccess]] = OMIT,
        additional_data: typing.Optional[AdditionalData] = OMIT,
        email: typing.Optional[Email] = OMIT,
        language: typing.Optional[Language] = OMIT,
        mfa_data: typing.Optional[MfaData] = OMIT,
        name: typing.Optional[NameUser] = OMIT,
        phone: typing.Optional[PhoneNumber] = OMIT,
        pwd: typing.Optional[str] = OMIT,
        scope: typing.Optional[typing.Sequence[OrgScope]] = OMIT,
        time_zone: typing.Optional[Timezone] = OMIT,
        usr_status: typing.Optional[UsrStatus] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[PayabliApiResponse]:
        """
        Parameters
        ----------
        user_id : int
            User Identifier

        access : typing.Optional[typing.Sequence[UsrAccess]]

        additional_data : typing.Optional[AdditionalData]

        email : typing.Optional[Email]
            The user's email address.

        language : typing.Optional[Language]

        mfa_data : typing.Optional[MfaData]

        name : typing.Optional[NameUser]

        phone : typing.Optional[PhoneNumber]
            The user's phone number.

        pwd : typing.Optional[str]

        scope : typing.Optional[typing.Sequence[OrgScope]]

        time_zone : typing.Optional[Timezone]

        usr_status : typing.Optional[UsrStatus]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PayabliApiResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"User/{jsonable_encoder(user_id)}",
            method="PUT",
            json={
                "access": convert_and_respect_annotation_metadata(
                    object_=access, annotation=typing.Sequence[UsrAccess], direction="write"
                ),
                "additionalData": additional_data,
                "email": email,
                "language": language,
                "mfaData": convert_and_respect_annotation_metadata(
                    object_=mfa_data, annotation=MfaData, direction="write"
                ),
                "name": name,
                "phone": phone,
                "pwd": pwd,
                "scope": convert_and_respect_annotation_metadata(
                    object_=scope, annotation=typing.Sequence[OrgScope], direction="write"
                ),
                "timeZone": time_zone,
                "usrStatus": usr_status,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PayabliApiResponse,
                    parse_obj_as(
                        type_=PayabliApiResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_user(
        self,
        user_id: int,
        *,
        entry: typing.Optional[str] = None,
        level: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[UserQueryRecord]:
        """
        Parameters
        ----------
        user_id : int
            The Payabli-generated `userId` value.

        entry : typing.Optional[str]
            The entrypoint identifier.

        level : typing.Optional[int]
            Entry level: 0 - partner, 2 - paypoint

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[UserQueryRecord]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"User/{jsonable_encoder(user_id)}",
            method="GET",
            params={
                "entry": entry,
                "level": level,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UserQueryRecord,
                    parse_obj_as(
                        type_=UserQueryRecord,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def logout_user(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[LogoutUserResponse]:
        """
        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[LogoutUserResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "User/authlogout",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    LogoutUserResponse,
                    parse_obj_as(
                        type_=LogoutUserResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def resend_mfa_code(
        self, entry: str, entry_type: int, usrname: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[PayabliApiResponseMfaBasic]:
        """
        Parameters
        ----------
        entry : str


        entry_type : int


        usrname : str


        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PayabliApiResponseMfaBasic]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"User/resendmfa/{jsonable_encoder(usrname)}/{jsonable_encoder(entry)}/{jsonable_encoder(entry_type)}",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PayabliApiResponseMfaBasic,
                    parse_obj_as(
                        type_=PayabliApiResponseMfaBasic,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def validate_mfa_user(
        self,
        *,
        mfa_code: typing.Optional[str] = OMIT,
        mfa_validation_code: typing.Optional[MfaValidationCode] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[PayabliApiResponseUserMfa]:
        """
        Parameters
        ----------
        mfa_code : typing.Optional[str]

        mfa_validation_code : typing.Optional[MfaValidationCode]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PayabliApiResponseUserMfa]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "User/mfa",
            method="POST",
            json={
                "mfaCode": mfa_code,
                "mfaValidationCode": mfa_validation_code,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PayabliApiResponseUserMfa,
                    parse_obj_as(
                        type_=PayabliApiResponseUserMfa,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from ...types.ach_holder_type import AchHolderType
from ...types.ach_sec_code import AchSecCode
from ...types.bin_data import BinData
from ...types.descriptor import Descriptor
from ...types.holdername import Holdername
from ...types.last_modified import LastModified
from ...types.maskedaccount import Maskedaccount
from .get_method_response_response_data_customers_item import GetMethodResponseResponseDataCustomersItem


class GetMethodResponseResponseData(UniversalBaseModel):
    aba: typing.Optional[str] = pydantic.Field(default=None)
    """
    Bank routing number
    """

    ach_holder_type: typing_extensions.Annotated[
        typing.Optional[AchHolderType], FieldMetadata(alias="achHolderType")
    ] = None
    ach_sec_code: typing_extensions.Annotated[typing.Optional[AchSecCode], FieldMetadata(alias="achSecCode")] = None
    bin: typing.Optional[str] = pydantic.Field(default=None)
    """
    The bank identification number (BIN)
    """

    bin_data: typing_extensions.Annotated[typing.Optional[BinData], FieldMetadata(alias="binData")] = None
    customers: typing.Optional[typing.List[GetMethodResponseResponseDataCustomersItem]] = None
    descriptor: typing.Optional[Descriptor] = None
    exp_date: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="expDate")] = pydantic.Field(
        default=None
    )
    """
    Expiration date for card in stored method in format MM/YY
    """

    holder_name: typing_extensions.Annotated[typing.Optional[Holdername], FieldMetadata(alias="holderName")] = (
        pydantic.Field(default=None)
    )
    """
    Account holder name in stored method
    """

    id_pmethod: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="idPmethod")] = pydantic.Field(
        default=None
    )
    """
    The stored payment method's identifier in Payabli
    """

    last_updated: typing_extensions.Annotated[typing.Optional[LastModified], FieldMetadata(alias="lastUpdated")] = (
        pydantic.Field(default=None)
    )
    """
    Timestamp for last update of stored method, in UTC
    """

    masked_account: typing_extensions.Annotated[
        typing.Optional[Maskedaccount], FieldMetadata(alias="maskedAccount")
    ] = None
    method: typing.Optional[str] = pydantic.Field(default=None)
    """
    The saved method's type: `card` or `ach`. 
    """

    method_type: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="methodType")] = pydantic.Field(
        default=None
    )
    """
    The payment method's token type
    """

    postal_code: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="postalCode")] = pydantic.Field(
        default=None
    )
    """
    The payment method postal code
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .customer_query_records_customer_consent_e_communication import CustomerQueryRecordsCustomerConsentECommunication
from .customer_query_records_customer_consent_sms import CustomerQueryRecordsCustomerConsentSms


class CustomerQueryRecordsCustomerConsent(UniversalBaseModel):
    e_communication: typing_extensions.Annotated[
        typing.Optional[CustomerQueryRecordsCustomerConsentECommunication], FieldMetadata(alias="eCommunication")
    ] = pydantic.Field(default=None)
    """
    Describes the customer's email communications consent status.
    """

    sms: typing.Optional[CustomerQueryRecordsCustomerConsentSms] = pydantic.Field(default=None)
    """
    Describes the customer's SMS communications consent status.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from ...types.ach_holder import AchHolder
from ...types.ach_holder_type import AchHolderType
from ...types.ach_sec_code import AchSecCode
from ...types.achaccount import Achaccount
from ...types.achaccounttype import Achaccounttype
from ...types.achrouting import Achrouting
from ...types.device import Device


class TokenizeAch(UniversalBaseModel):
    method: str = pydantic.Field()
    """
    The type of payment method to tokenize. For ACH, this is always `ach`.
    """

    ach_account: typing_extensions.Annotated[Achaccount, FieldMetadata(alias="achAccount")]
    ach_account_type: typing_extensions.Annotated[Achaccounttype, FieldMetadata(alias="achAccountType")]
    ach_code: typing_extensions.Annotated[typing.Optional[AchSecCode], FieldMetadata(alias="achCode")] = None
    ach_holder: typing_extensions.Annotated[AchHolder, FieldMetadata(alias="achHolder")]
    ach_holder_type: typing_extensions.Annotated[
        typing.Optional[AchHolderType], FieldMetadata(alias="achHolderType")
    ] = None
    ach_routing: typing_extensions.Annotated[Achrouting, FieldMetadata(alias="achRouting")]
    device: typing.Optional[Device] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ...types.application_data import ApplicationData
from ...types.application_data_managed import ApplicationDataManaged
from ...types.application_data_odp import ApplicationDataOdp
from ...types.application_data_pay_in import ApplicationDataPayIn

AddApplicationRequest = typing.Union[ApplicationDataPayIn, ApplicationDataManaged, ApplicationDataOdp, ApplicationData]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class DisplayProperty(UniversalBaseModel):
    display: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When `true`, the field is displayed on the receipt.
    """

    fixed: typing_extensions.Annotated[typing.Optional[bool], FieldMetadata(alias="Fixed")] = pydantic.Field(
        default=None
    )
    """
    This field is unused.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The field's name.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .additional_data_string import AdditionalDataString
from .annualrevenue import Annualrevenue
from .application_data_pay_in_bank_data import ApplicationDataPayInBankData
from .application_data_pay_in_contacts_item import ApplicationDataPayInContactsItem
from .application_data_pay_in_ownership_item import ApplicationDataPayInOwnershipItem
from .application_data_pay_in_services import ApplicationDataPayInServices
from .avgmonthly import Avgmonthly
from .baddress_1 import Baddress1
from .baddress_2 import Baddress2
from .bcity import Bcity
from .bcountry import Bcountry
from .binperson import Binperson
from .binphone import Binphone
from .binweb import Binweb
from .boarding_average_bill_size import BoardingAverageBillSize
from .boarding_avg_monthly_bill import BoardingAvgMonthlyBill
from .bstate import Bstate
from .bsummary import Bsummary
from .busstartdate import Busstartdate
from .bzip import Bzip
from .dbaname import Dbaname
from .ein import Ein
from .email import Email
from .external_paypoint_id import ExternalPaypointId
from .fax_number import FaxNumber
from .highticketamt import Highticketamt
from .legalname import Legalname
from .license import License
from .licensestate import Licensestate
from .maddress import Maddress
from .maddress_1 import Maddress1
from .mcc import Mcc
from .mcity import Mcity
from .mcountry import Mcountry
from .mstate import Mstate
from .mzip import Mzip
from .orgid import Orgid
from .own_type import OwnType
from .phone_number import PhoneNumber
from .recipient_email_notification import RecipientEmailNotification
from .rep_code import RepCode
from .rep_name import RepName
from .rep_office import RepOffice
from .resumable import Resumable
from .signer_data_request import SignerDataRequest
from .taxfillname import Taxfillname
from .template_id import TemplateId
from .ticketamt import Ticketamt
from .website import Website
from .whencharged import Whencharged
from .whendelivered import Whendelivered
from .whenprovided import Whenprovided
from .whenrefunded import Whenrefunded


class ApplicationDataPayIn(UniversalBaseModel):
    """
    Fields for Pay In boarding applications.
    """

    services: ApplicationDataPayInServices
    annual_revenue: typing_extensions.Annotated[
        typing.Optional[Annualrevenue], FieldMetadata(alias="annualRevenue")
    ] = None
    average_bill_size: typing_extensions.Annotated[
        typing.Optional[BoardingAverageBillSize], FieldMetadata(alias="averageBillSize")
    ] = None
    average_monthly_bill: typing_extensions.Annotated[
        typing.Optional[BoardingAvgMonthlyBill], FieldMetadata(alias="averageMonthlyBill")
    ] = None
    avgmonthly: typing.Optional[Avgmonthly] = None
    baddress: typing.Optional[Baddress1] = None
    baddress_1: typing_extensions.Annotated[typing.Optional[Baddress2], FieldMetadata(alias="baddress1")] = None
    bank_data: typing_extensions.Annotated[ApplicationDataPayInBankData, FieldMetadata(alias="bankData")]
    bcity: typing.Optional[Bcity] = None
    bcountry: typing.Optional[Bcountry] = None
    binperson: typing.Optional[Binperson] = None
    binphone: typing.Optional[Binphone] = None
    binweb: typing.Optional[Binweb] = None
    boarding_link_id: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="boardingLinkId")] = (
        pydantic.Field(default=None)
    )
    """
    Boarding link ID for the application. Either `templateId` or `boardingLinkId` are required.
    """

    bstate: typing.Optional[Bstate] = None
    bsummary: typing.Optional[Bsummary] = None
    btype: typing.Optional[OwnType] = None
    bzip: typing.Optional[Bzip] = None
    contacts: typing.Optional[typing.List[ApplicationDataPayInContactsItem]] = pydantic.Field(default=None)
    """
    List of contacts for the business.
    """

    credit_limit: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="creditLimit")] = (
        pydantic.Field(default=None)
    )
    """
    The maximum amount of credit that our lending partner, has authorized to your business. It's the upper boundary on how much you can spend or owe on a credit account at any given time.
    """

    dba_name: typing_extensions.Annotated[typing.Optional[Dbaname], FieldMetadata(alias="dbaName")] = pydantic.Field(
        default=None
    )
    """
    The alternate or common name that this business is doing business under usually referred to as a DBA name. Payabli strongly recommends including this information.
    """

    ein: typing.Optional[Ein] = None
    externalpaypoint_id: typing_extensions.Annotated[
        typing.Optional[ExternalPaypointId], FieldMetadata(alias="externalpaypointID")
    ] = None
    faxnumber: typing.Optional[FaxNumber] = pydantic.Field(default=None)
    """
    The business's fax number.
    """

    highticketamt: typing.Optional[Highticketamt] = None
    legal_name: typing_extensions.Annotated[typing.Optional[Legalname], FieldMetadata(alias="legalName")] = None
    license: typing.Optional[License] = None
    licstate: typing.Optional[Licensestate] = None
    maddress: typing.Optional[Maddress] = None
    maddress_1: typing_extensions.Annotated[typing.Optional[Maddress1], FieldMetadata(alias="maddress1")] = None
    mcc: typing.Optional[Mcc] = None
    mcity: typing.Optional[Mcity] = None
    mcountry: typing.Optional[Mcountry] = None
    mstate: typing.Optional[Mstate] = None
    mzip: typing.Optional[Mzip] = None
    org_id: typing_extensions.Annotated[typing.Optional[Orgid], FieldMetadata(alias="orgId")] = None
    ownership: typing.Optional[typing.List[ApplicationDataPayInOwnershipItem]] = pydantic.Field(default=None)
    """
    List of Owners with at least a 25% ownership.
    """

    phonenumber: PhoneNumber = pydantic.Field()
    """
    The business's phone number.
    """

    processing_region: typing_extensions.Annotated[str, FieldMetadata(alias="processingRegion")] = pydantic.Field()
    """
    The business's processing region, either `US` or `CA`.
    """

    recipient_email: typing_extensions.Annotated[typing.Optional[Email], FieldMetadata(alias="recipientEmail")] = (
        pydantic.Field(default=None)
    )
    """
    Email address for the applicant. This is used to send the applicant a boarding link.
    """

    recipient_email_notification: typing_extensions.Annotated[
        typing.Optional[RecipientEmailNotification], FieldMetadata(alias="recipientEmailNotification")
    ] = None
    resumable: typing.Optional[Resumable] = None
    signer: SignerDataRequest
    startdate: typing.Optional[Busstartdate] = None
    tax_fill_name: typing_extensions.Annotated[typing.Optional[Taxfillname], FieldMetadata(alias="taxFillName")] = None
    template_id: typing_extensions.Annotated[typing.Optional[TemplateId], FieldMetadata(alias="templateId")] = (
        pydantic.Field(default=None)
    )
    """
    The associated boarding template's ID in Payabli. Either `templateId` or `boardingLinkId` are required.
    """

    ticketamt: typing.Optional[Ticketamt] = None
    website: typing.Optional[Website] = None
    when_charged: typing_extensions.Annotated[Whencharged, FieldMetadata(alias="whenCharged")]
    when_delivered: typing_extensions.Annotated[Whendelivered, FieldMetadata(alias="whenDelivered")]
    when_provided: typing_extensions.Annotated[Whenprovided, FieldMetadata(alias="whenProvided")]
    when_refunded: typing_extensions.Annotated[Whenrefunded, FieldMetadata(alias="whenRefunded")]
    additional_data: typing_extensions.Annotated[
        typing.Optional[AdditionalDataString], FieldMetadata(alias="additionalData")
    ] = None
    rep_code: typing_extensions.Annotated[typing.Optional[RepCode], FieldMetadata(alias="RepCode")] = None
    rep_name: typing_extensions.Annotated[typing.Optional[RepName], FieldMetadata(alias="RepName")] = None
    rep_office: typing_extensions.Annotated[typing.Optional[RepOffice], FieldMetadata(alias="RepOffice")] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
